unit MainFormUnit;

interface

uses
  Classes,
  Controls,
  Dialogs,
  ExtCtrls,
  Forms,
  IniFiles,
  Menus,
  Messages,
  StdCtrls,
  Windows,
  RunUOControllerUnit, JvComponent, JvTrayIcon;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    StartItem: TMenuItem;
    FileSeparator: TMenuItem;
    ExitItem: TMenuItem;
    HelpMenu: TMenuItem;
    AboutItem: TMenuItem;
    OptionsMenu: TMenuItem;
    FontItem: TMenuItem;
    WebsiteItem: TMenuItem;
    ForumsItem: TMenuItem;
    IRCItem: TMenuItem;
    AboutSeparator: TMenuItem;
    StartServerAutomaticallyItem: TMenuItem;
    FontDialog: TFontDialog;
    ExecutableOpenDialog: TOpenDialog;
    RunUOLocationItem: TMenuItem;
    OptionsSeparator: TMenuItem;
    BottomPanel: TPanel;
    SendEdit: TEdit;
    SendButton: TButton;
    StopItem: TMenuItem;
    TopPanel: TPanel;
    MainMemo: TMemo;
    EditMenu: TMenuItem;
    CopyItem: TMenuItem;
    EditSeparator: TMenuItem;
    ClearItem: TMenuItem;
    JvTrayIcon1: TJvTrayIcon;
    procedure AboutItemClick(Sender: TObject);
    procedure FontItemClick(Sender: TObject);
    procedure StartItemClick(Sender: TObject);
    procedure RunUOLocationItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SendButtonClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure StopItemClick(Sender: TObject);
    procedure WebsiteItemClick(Sender: TObject);
    procedure ForumsItemClick(Sender: TObject);
    procedure IRCItemClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SendEditKeyPress(Sender: TObject; var Key: Char);
    procedure CopyItemClick(Sender: TObject);
    procedure ClearItemClick(Sender: TObject);
  private
    FController: TRunUOController;
    FSettings: TMemIniFile;
    procedure CatchErrorString(sender: TObject; text: String);
    procedure CatchOutputString(sender: TObject; text: String);
    procedure WriteString(text: String);
    procedure StartServer;
    procedure StopServer;
  public
    constructor Create(AOwner: TComponent); override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  SysUtils, TypInfo, ShellAPI, Clipbrd, AboutBoxUnit;

resourcestring
  sIniFile='RunUOWrapper.ini';
  sServerSection='Server';
  sExecutablePathKey='ExePath';
  sWindowSection='Window';
  sFontNameKey='FontName';
  sFontSizeKey='FontSize';
  sFontStyleKey='FontStyle';
  sStartFormat='Starting RunUO in %s...'#13#10;
  sStartFormatBegin='Starting RunUO in ';
  sStartFormatEnd='...'#13#10;
  sStopFormat='Stopping RunUO in %s...'#13#10;
  sStopFormatBegin='Stopping RunUO in ';
  sStopFormatEnd='...'#13#10;
  sDoneMessage='Done.'#13#10;
  sConfirmExit='Do you want to exit and shut down RunUO?';
  sConfirmStop='Are you sure you want to shut down RunUO?';
  sWebsite='http://www.runuo.com/';
  sForum='http://www.runuo.com/forum/';
  sIRC='irc://irc.coldfront.net/runuo';

procedure TMainForm.AboutItemClick(Sender: TObject);
begin
  TAboutBox.Display;
end;

procedure TMainForm.FontItemClick(Sender: TObject);
begin
  FontDialog.Font.Assign(MainMemo.Font);
  if FontDialog.Execute then
  begin
    MainMemo.Font.Assign(FontDialog.Font);
    FSettings.WriteString(sWindowSection, sFontNameKey, MainMemo.Font.Name);
    FSettings.WriteInteger(sWindowSection, sFontSizeKey, MainMemo.Font.Size);
    FSettings.WriteString(sWindowSection, sFontStyleKey, GetSetProp(MainMemo.Font,
      'Style', True));
    FSettings.UpdateFile;
  end;
end;

procedure TMainForm.StartItemClick(Sender: TObject);
var
  app: String;
begin
  app:=FSettings.ReadString(sServerSection, sExecutablePathKey, '');
  while (app='') or not FileExists(app) do
  begin
    if ExecutableOpenDialog.Execute then
      if FileExists(ExecutableOpenDialog.FileName) then
      begin
        FSettings.WriteString(sServerSection, sExecutablePathKey,
          ExecutableOpenDialog.FileName);
        FSettings.UpdateFile;
        app:=FSettings.ReadString(sServerSection, sExecutablePathKey, '');
      end
      else
    else
    begin
      Exit;
    end;
  end;
  FController:=TRunUOController.Create(app, ExtractFilePath(app));
  FController.OnStandardOutput:=CatchOutputString;
  FController.OnStandardError:=CatchErrorString;
  StartServer;
  StartItem.Enabled:=False;
  StopItem.Enabled:=True;
end;

procedure TMainForm.RunUOLocationItemClick(Sender: TObject);
begin
  if ExecutableOpenDialog.Execute then
  begin
    FSettings.WriteString(sServerSection, sExecutablePathKey,
      ExecutableOpenDialog.FileName);
    FSettings.UpdateFile;
  end;
end;

procedure TMainForm.StartServer;
begin
  if Assigned(FController) then
  begin
    WriteString(sStartFormatBegin+FController.AppName+sStartFormatEnd);
    FController.Start;
  end;
end;

procedure TMainForm.StopServer;
begin
  if Assigned(FController) then
  begin
    WriteString(sStopFormatBegin+FController.AppName+sStopFormatEnd);
    FController.Stop;
    WriteString(sDoneMessage);
  end;
end;

procedure TMainForm.WriteString(text: String);
begin
  MainMemo.SelStart:=-1;
  MainMemo.SelLength:=0;
  MainMemo.SelText:=text;
  CopyItem.Enabled:=True;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  StopServer;
end;

procedure TMainForm.SendButtonClick(Sender: TObject);
begin
  if Assigned(FController) then
  begin
    FController.SendString(SendEdit.Text+#13#10);
    WriteString(SendEdit.Text+#13#10);
  end;
  SendEdit.Clear;
  SendEdit.SetFocus;
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Assigned(FController) and FController.Running then
    CanClose:=MessageDlg(sConfirmExit,
      mtConfirmation, [mbYes,mbNo], 0) = mrYes
end;

procedure TMainForm.StopItemClick(Sender: TObject);
begin
  if MessageDlg(sConfirmStop, mtConfirmation,
    [mbYes,mbNo], 0) = mrYes then
  begin
    StopServer;
    StartItem.Enabled:=True;
    StopItem.Enabled:=False;
  end;
end;

procedure TMainForm.WebsiteItemClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar(sWebsite), nil, nil, SW_SHOW);
end;

procedure TMainForm.ForumsItemClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar(sForum), nil, nil, SW_SHOW);
end;

procedure TMainForm.IRCItemClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar(sIRC), nil, nil, SW_SHOW);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FSettings.UpdateFile;
  FSettings.Free;
end;

constructor TMainForm.Create(AOwner: TComponent);
begin
  inherited;
  Font.Assign(Screen.MenuFont);
  FController:=nil;
  FSettings:=TMemIniFile.Create(ExtractFilePath(ParamStr(0))+sIniFile);
  FSettings.CaseSensitive:=False;
  MainMemo.Font.Name:=FSettings.ReadString(sWindowSection, sFontNameKey,
    'Courier New');
  MainMemo.Font.Size:=FSettings.ReadInteger(sWindowSection, sFontSizeKey, 8);
  SetSetProp(MainMemo.Font, 'Style', FSettings.ReadString(sWindowSection,
    sFontStyleKey, '[]'));
  FSettings.WriteString(sWindowSection, sFontNameKey, MainMemo.Font.Name);
  FSettings.WriteInteger(sWindowSection, sFontSizeKey, MainMemo.Font.Size);
  FSettings.WriteString(sWindowSection, sFontStyleKey, GetSetProp(MainMemo.Font,
    'Style', True));
end;

procedure TMainForm.SendEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then
  begin
    SendButton.Click;
    Key:=#0;
  end;
end;

procedure TMainForm.CopyItemClick(Sender: TObject);
begin
  Clipboard.AsText:=MainMemo.Text;
{  MainMemo.SelectAll;
  MainMemo.CopyToClipboard;
  MainMemo.SelStart:=-1;}
end;

procedure TMainForm.ClearItemClick(Sender: TObject);
begin
  MainMemo.Clear;
  CopyItem.Enabled:=False;
end;

procedure TMainForm.CatchErrorString(sender: TObject; text: String);
begin
  WriteString('ERROR: '+text);
end;

procedure TMainForm.CatchOutputString(sender: TObject; text: String);
begin
  WriteString(text);
end;

end.
